# -*- coding: utf-8 -*-
from uuid import uuid4

import django.core.validators
import django.utils.timezone
import django_fsm
import model_utils.fields
import waldur_core.core.shims
from django.conf import settings
from django.db import migrations, models

import waldur_core.core.fields
import waldur_core.core.validators
import waldur_core.media.validators
import waldur_core.media.models
import waldur_core.structure.models


def create_quotas(apps, schema_editor):
    Project = apps.get_model('structure', 'Project')
    Customer = apps.get_model('structure', 'Customer')
    Quota = apps.get_model('quotas', 'Quota')

    # We can not use model constants in migrations because they can be changed in future
    quota_name_map = {
        Project: 'nc_global_project_count',
        Customer: 'nc_global_customer_count',
    }

    for model in [Project, Customer]:
        name = quota_name_map[model]
        usage = model.objects.count()
        if not Quota.objects.filter(name=name, object_id__isnull=True).exists():
            Quota.objects.create(uuid=uuid4().hex, name=name, usage=usage)
        else:
            Quota.objects.filter(name=name, object_id__isnull=True).update(usage=usage)


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('quotas', '0001_squashed_0004'),
        ('taggit', '0002_auto_20150616_2121'),
        ('contenttypes', '0002_remove_content_type_name'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Customer',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('name', models.CharField(max_length=150, validators=[waldur_core.core.validators.validate_name], verbose_name='name')),
                ('uuid', waldur_core.core.fields.UUIDField()),
                ('image', models.ImageField(blank=True, null=True, upload_to=waldur_core.media.models.get_upload_path)),
                ('vat_code', models.CharField(blank=True, help_text='VAT number', max_length=20)),
                ('vat_name', models.CharField(blank=True, help_text='Optional business name retrieved for the VAT number.', max_length=255)),
                ('vat_address', models.CharField(blank=True, help_text='Optional business address retrieved for the VAT number.', max_length=255)),
                ('is_company', models.BooleanField(default=False, help_text='Is company or private person')),
                ('country', waldur_core.core.fields.CountryField(blank=True, choices=[('AF', 'Afghanistan'), ('AX', '\xc5land Islands'), ('AL', 'Albania'), ('DZ', 'Algeria'), ('AS', 'American Samoa'), ('AD', 'Andorra'), ('AO', 'Angola'), ('AI', 'Anguilla'), ('AQ', 'Antarctica'), ('AG', 'Antigua and Barbuda'), ('AR', 'Argentina'), ('AM', 'Armenia'), ('AW', 'Aruba'), ('AU', 'Australia'), ('AT', 'Austria'), ('AZ', 'Azerbaijan'), ('BS', 'Bahamas'), ('BH', 'Bahrain'), ('BD', 'Bangladesh'), ('BB', 'Barbados'), ('BY', 'Belarus'), ('BE', 'Belgium'), ('BZ', 'Belize'), ('BJ', 'Benin'), ('BM', 'Bermuda'), ('BT', 'Bhutan'), ('BO', 'Bolivia, Plurinational State of'), ('BQ', 'Bonaire, Sint Eustatius and Saba'), ('BA', 'Bosnia and Herzegovina'), ('BW', 'Botswana'), ('BV', 'Bouvet Island'), ('BR', 'Brazil'), ('IO', 'British Indian Ocean Territory'), ('BN', 'Brunei Darussalam'), ('BG', 'Bulgaria'), ('BF', 'Burkina Faso'), ('BI', 'Burundi'), ('KH', 'Cambodia'), ('CM', 'Cameroon'), ('CA', 'Canada'), ('CV', 'Cape Verde'), ('KY', 'Cayman Islands'), ('CF', 'Central African Republic'), ('TD', 'Chad'), ('CL', 'Chile'), ('CN', 'China'), ('CX', 'Christmas Island'), ('CC', 'Cocos (Keeling) Islands'), ('CO', 'Colombia'), ('KM', 'Comoros'), ('CG', 'Congo'), ('CD', 'Congo, The Democratic Republic of the'), ('CK', 'Cook Islands'), ('CR', 'Costa Rica'), ('CI', "C\xf4te d'Ivoire"), ('HR', 'Croatia'), ('CU', 'Cuba'), ('CW', 'Cura\xe7ao'), ('CY', 'Cyprus'), ('CZ', 'Czech Republic'), ('DK', 'Denmark'), ('DJ', 'Djibouti'), ('DM', 'Dominica'), ('DO', 'Dominican Republic'), ('EC', 'Ecuador'), ('EG', 'Egypt'), ('SV', 'El Salvador'), ('GQ', 'Equatorial Guinea'), ('ER', 'Eritrea'), ('EE', 'Estonia'), ('ET', 'Ethiopia'), ('FK', 'Falkland Islands (Malvinas)'), ('FO', 'Faroe Islands'), ('FJ', 'Fiji'), ('FI', 'Finland'), ('FR', 'France'), ('GF', 'French Guiana'), ('PF', 'French Polynesia'), ('TF', 'French Southern Territories'), ('GA', 'Gabon'), ('GM', 'Gambia'), ('GE', 'Georgia'), ('DE', 'Germany'), ('GH', 'Ghana'), ('GI', 'Gibraltar'), ('GR', 'Greece'), ('GL', 'Greenland'), ('GD', 'Grenada'), ('GP', 'Guadeloupe'), ('GU', 'Guam'), ('GT', 'Guatemala'), ('GG', 'Guernsey'), ('GN', 'Guinea'), ('GW', 'Guinea-Bissau'), ('GY', 'Guyana'), ('HT', 'Haiti'), ('HM', 'Heard Island and McDonald Islands'), ('VA', 'Holy See (Vatican City State)'), ('HN', 'Honduras'), ('HK', 'Hong Kong'), ('HU', 'Hungary'), ('IS', 'Iceland'), ('IN', 'India'), ('ID', 'Indonesia'), ('IR', 'Iran, Islamic Republic of'), ('IQ', 'Iraq'), ('IE', 'Ireland'), ('IM', 'Isle of Man'), ('IL', 'Israel'), ('IT', 'Italy'), ('JM', 'Jamaica'), ('JP', 'Japan'), ('JE', 'Jersey'), ('JO', 'Jordan'), ('KZ', 'Kazakhstan'), ('KE', 'Kenya'), ('KI', 'Kiribati'), ('KP', "Korea, Democratic People's Republic of"), ('KR', 'Korea, Republic of'), ('KW', 'Kuwait'), ('KG', 'Kyrgyzstan'), ('LA', "Lao People's Democratic Republic"), ('LV', 'Latvia'), ('LB', 'Lebanon'), ('LS', 'Lesotho'), ('LR', 'Liberia'), ('LY', 'Libya'), ('LI', 'Liechtenstein'), ('LT', 'Lithuania'), ('LU', 'Luxembourg'), ('MO', 'Macao'), ('MK', 'Macedonia, Republic of'), ('MG', 'Madagascar'), ('MW', 'Malawi'), ('MY', 'Malaysia'), ('MV', 'Maldives'), ('ML', 'Mali'), ('MT', 'Malta'), ('MH', 'Marshall Islands'), ('MQ', 'Martinique'), ('MR', 'Mauritania'), ('MU', 'Mauritius'), ('YT', 'Mayotte'), ('MX', 'Mexico'), ('FM', 'Micronesia, Federated States of'), ('MD', 'Moldova, Republic of'), ('MC', 'Monaco'), ('MN', 'Mongolia'), ('ME', 'Montenegro'), ('MS', 'Montserrat'), ('MA', 'Morocco'), ('MZ', 'Mozambique'), ('MM', 'Myanmar'), ('NA', 'Namibia'), ('NR', 'Nauru'), ('NP', 'Nepal'), ('NL', 'Netherlands'), ('NC', 'New Caledonia'), ('NZ', 'New Zealand'), ('NI', 'Nicaragua'), ('NE', 'Niger'), ('NG', 'Nigeria'), ('NU', 'Niue'), ('NF', 'Norfolk Island'), ('MP', 'Northern Mariana Islands'), ('NO', 'Norway'), ('OM', 'Oman'), ('PK', 'Pakistan'), ('PW', 'Palau'), ('PS', 'Palestine, State of'), ('PA', 'Panama'), ('PG', 'Papua New Guinea'), ('PY', 'Paraguay'), ('PE', 'Peru'), ('PH', 'Philippines'), ('PN', 'Pitcairn'), ('PL', 'Poland'), ('PT', 'Portugal'), ('PR', 'Puerto Rico'), ('QA', 'Qatar'), ('RE', 'R\xe9union'), ('RO', 'Romania'), ('RU', 'Russian Federation'), ('RW', 'Rwanda'), ('BL', 'Saint Barth\xe9lemy'), ('SH', 'Saint Helena, Ascension and Tristan da Cunha'), ('KN', 'Saint Kitts and Nevis'), ('LC', 'Saint Lucia'), ('MF', 'Saint Martin (French part)'), ('PM', 'Saint Pierre and Miquelon'), ('VC', 'Saint Vincent and the Grenadines'), ('WS', 'Samoa'), ('SM', 'San Marino'), ('ST', 'Sao Tome and Principe'), ('SA', 'Saudi Arabia'), ('SN', 'Senegal'), ('RS', 'Serbia'), ('SC', 'Seychelles'), ('SL', 'Sierra Leone'), ('SG', 'Singapore'), ('SX', 'Sint Maarten (Dutch part)'), ('SK', 'Slovakia'), ('SI', 'Slovenia'), ('SB', 'Solomon Islands'), ('SO', 'Somalia'), ('ZA', 'South Africa'), ('GS', 'South Georgia and the South Sandwich Islands'), ('ES', 'Spain'), ('LK', 'Sri Lanka'), ('SD', 'Sudan'), ('SR', 'Suriname'), ('SS', 'South Sudan'), ('SJ', 'Svalbard and Jan Mayen'), ('SZ', 'Swaziland'), ('SE', 'Sweden'), ('CH', 'Switzerland'), ('SY', 'Syrian Arab Republic'), ('TW', 'Taiwan, Province of China'), ('TJ', 'Tajikistan'), ('TZ', 'Tanzania, United Republic of'), ('TH', 'Thailand'), ('TL', 'Timor-Leste'), ('TG', 'Togo'), ('TK', 'Tokelau'), ('TO', 'Tonga'), ('TT', 'Trinidad and Tobago'), ('TN', 'Tunisia'), ('TR', 'Turkey'), ('TM', 'Turkmenistan'), ('TC', 'Turks and Caicos Islands'), ('TV', 'Tuvalu'), ('UG', 'Uganda'), ('UA', 'Ukraine'), ('AE', 'United Arab Emirates'), ('GB', 'United Kingdom'), ('US', 'United States'), ('UM', 'United States Minor Outlying Islands'), ('UY', 'Uruguay'), ('UZ', 'Uzbekistan'), ('VU', 'Vanuatu'), ('VE', 'Venezuela, Bolivarian Republic of'), ('VN', 'Viet Nam'), ('VG', 'Virgin Islands, British'), ('VI', 'Virgin Islands, U.S.'), ('WF', 'Wallis and Futuna'), ('EH', 'Western Sahara'), ('YE', 'Yemen'), ('ZM', 'Zambia'), ('ZW', 'Zimbabwe')], max_length=2)),
                ('native_name', models.CharField(blank=True, default='', max_length=160)),
                ('abbreviation', models.CharField(blank=True, max_length=12)),
                ('contact_details', models.TextField(blank=True, validators=[django.core.validators.MaxLengthValidator(500)])),
                ('agreement_number', models.PositiveIntegerField(blank=True, null=True, unique=True)),
                ('email', models.EmailField(blank=True, max_length=75, verbose_name='email address')),
                ('phone_number', models.CharField(blank=True, max_length=255, verbose_name='phone number')),
                ('access_subnets', models.TextField(blank=True, default='', help_text='Enter a comma separated list of IPv4 or IPv6 CIDR addresses from where connection to self-service is allowed.', validators=[waldur_core.core.validators.validate_cidr_list])),
                ('registration_code', models.CharField(blank=True, default='', max_length=160)),
                ('type', models.CharField(blank=True, max_length=150)),
                ('address', models.CharField(blank=True, max_length=300)),
                ('postal', models.CharField(blank=True, max_length=20)),
                ('bank_name', models.CharField(blank=True, max_length=150)),
                ('bank_account', models.CharField(blank=True, max_length=50)),
                ('accounting_start_date', models.DateTimeField(default=django.utils.timezone.now, verbose_name='Start date of accounting')),
                ('default_tax_percent', models.DecimalField(decimal_places=2, default=0, max_digits=4, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(100)])),
            ],
            options={
                'verbose_name': 'organization',
            },
            bases=(waldur_core.core.models.DescendantMixin, waldur_core.structure.models.PermissionMixin, waldur_core.logging.loggers.LoggableMixin, models.Model),
        ),
        migrations.CreateModel(
            name='CustomerPermission',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False)),
                ('expiration_time', models.DateTimeField(blank=True, null=True)),
                ('is_active', models.NullBooleanField(db_index=True, default=True)),
                ('role', waldur_core.structure.models.CustomerRole(choices=[('owner', 'Owner'), ('support', 'Support')], db_index=True, max_length=30)),
                ('created_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to=settings.AUTH_USER_MODEL)),
                ('customer', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='permissions', to='structure.Customer', verbose_name='organization')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='Project',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('description', models.CharField(blank=True, max_length=500, verbose_name='description')),
                ('name', models.CharField(max_length=150, validators=[waldur_core.core.validators.validate_name], verbose_name='name')),
                ('uuid', waldur_core.core.fields.UUIDField()),
            ],
            options={
                'abstract': False,
            },
            bases=(waldur_core.core.models.DescendantMixin, waldur_core.structure.models.PermissionMixin, waldur_core.structure.models.StructureLoggableMixin, models.Model),
        ),
        migrations.CreateModel(
            name='ProjectPermission',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', waldur_core.core.fields.UUIDField()),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False)),
                ('expiration_time', models.DateTimeField(blank=True, null=True)),
                ('is_active', models.NullBooleanField(db_index=True, default=True)),
                ('role', waldur_core.structure.models.ProjectRole(choices=[('admin', 'Administrator'), ('manager', 'Manager'), ('support', 'Support')], db_index=True, max_length=30)),
                ('created_by', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to=settings.AUTH_USER_MODEL)),
                ('project', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='permissions', to='structure.Project')),
                ('user', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
        ),
        migrations.CreateModel(
            name='ProjectType',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('description', models.CharField(blank=True, max_length=500, verbose_name='description')),
                ('name', models.CharField(max_length=150, validators=[waldur_core.core.validators.validate_name], verbose_name='name')),
                ('uuid', waldur_core.core.fields.UUIDField()),
            ],
            options={
                'ordering': ['name'],
                'verbose_name': 'Project type',
                'verbose_name_plural': 'Project types',
            },
        ),
        migrations.CreateModel(
            name='ServiceCertification',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('description', models.CharField(blank=True, max_length=500, verbose_name='description')),
                ('uuid', waldur_core.core.fields.UUIDField()),
                ('link', models.URLField(blank=True, max_length=255)),
                ('name', models.CharField(max_length=150, unique=True, validators=[waldur_core.core.validators.validate_name], verbose_name='name')),
            ],
            options={
                'ordering': ['-name'],
                'verbose_name': 'Service Certification',
                'verbose_name_plural': 'Service Certifications',
            },
        ),
        migrations.CreateModel(
            name='ServiceSettings',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=150, validators=[waldur_core.core.validators.validate_name], verbose_name='name')),
                ('uuid', waldur_core.core.fields.UUIDField()),
                ('error_message', models.TextField(blank=True)),
                ('state', django_fsm.FSMIntegerField(choices=[(5, 'Creation Scheduled'), (6, 'Creating'), (1, 'Update Scheduled'), (2, 'Updating'), (7, 'Deletion Scheduled'), (8, 'Deleting'), (3, 'OK'), (4, 'Erred')], default=5)),
                ('backend_url', waldur_core.core.fields.BackendURLField(blank=True, null=True)),
                ('username', models.CharField(blank=True, max_length=100, null=True)),
                ('password', models.CharField(blank=True, max_length=100, null=True)),
                ('domain', models.CharField(blank=True, max_length=200, null=True)),
                ('token', models.CharField(blank=True, max_length=255, null=True)),
                ('certificate', models.FileField(blank=True, null=True, upload_to='certs', validators=[
                    waldur_core.media.validators.FileTypeValidator(allowed_extensions=['pem'], allowed_types=['application/x-pem-file', 'application/x-x509-ca-cert', 'text/plain'])])),
                ('type', models.CharField(db_index=True, max_length=255, validators=[waldur_core.structure.models.validate_service_type])),
                ('options', waldur_core.core.fields.JSONField(blank=True, default={}, help_text='Extra options')),
                ('geolocations', waldur_core.core.fields.JSONField(blank=True, default=[], help_text='List of latitudes and longitudes. For example: [{"latitude": 123, "longitude": 345}, {"latitude": 456, "longitude": 678}]')),
                ('shared', models.BooleanField(default=False, help_text='Anybody can use it')),
                ('homepage', models.URLField(blank=True, max_length=255)),
                ('terms_of_services', models.URLField(blank=True, max_length=255)),
                ('object_id', models.PositiveIntegerField(null=True)),
                ('certifications', models.ManyToManyField(blank=True, related_name='service_settings', to='structure.ServiceCertification')),
                ('content_type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='contenttypes.ContentType')),
                ('customer', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='service_settings', to='structure.Customer', verbose_name='organization')),
                ('tags', waldur_core.core.shims.TaggableManager(related_name='+', blank=True, help_text='A comma-separated list of tags.', through='taggit.TaggedItem', to='taggit.Tag', verbose_name='Tags')),
            ],
            options={
                'verbose_name': 'Service settings',
                'verbose_name_plural': 'Service settings',
            },
            bases=(models.Model, waldur_core.logging.loggers.LoggableMixin),
        ),
        migrations.AddField(
            model_name='project',
            name='certifications',
            field=models.ManyToManyField(blank=True, related_name='projects', to='structure.ServiceCertification'),
        ),
        migrations.AddField(
            model_name='project',
            name='customer',
            field=models.ForeignKey(on_delete=django.db.models.deletion.PROTECT, related_name='projects', to='structure.Customer', verbose_name='organization'),
        ),
        migrations.AddField(
            model_name='project',
            name='type',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.PROTECT, to='structure.ProjectType', verbose_name='project type'),
        ),
        migrations.CreateModel(
            name='PrivateServiceSettings',
            fields=[
            ],
            options={
                'proxy': True,
                'verbose_name_plural': 'Private provider settings',
                'indexes': [],
            },
            bases=('structure.servicesettings',),
        ),
        migrations.CreateModel(
            name='SharedServiceSettings',
            fields=[
            ],
            options={
                'proxy': True,
                'verbose_name_plural': 'Shared provider settings',
                'indexes': [],
            },
            bases=('structure.servicesettings',),
        ),
        migrations.AlterUniqueTogether(
            name='projectpermission',
            unique_together=set([('project', 'role', 'user', 'is_active')]),
        ),
        migrations.AlterUniqueTogether(
            name='customerpermission',
            unique_together=set([('customer', 'role', 'user', 'is_active')]),
        ),
        migrations.RunPython(create_quotas),
    ]



from waldur_core.logging.loggers import EventLogger, event_logger

from .models import Issue, Comment


def get_issue_scopes(issue):
    project = issue.project.service_project_link.project
    result = {project, project.customer}
    if issue.resource:
        result.add(issue.resource)
    return result


class IssueEventLogger(EventLogger):
    issue = Issue

    class Meta:
        event_types = ('issue_deletion_succeeded',
                       'issue_update_succeeded',
                       'issue_creation_succeeded')
        event_groups = {
            'jira': event_types
        }

    @staticmethod
    def get_scopes(event_context):
        issue = event_context['issue']
        return get_issue_scopes(issue)


class CommentEventLogger(EventLogger):
    comment = Comment

    class Meta:
        event_types = ('comment_deletion_succeeded',
                       'comment_update_succeeded',
                       'comment_creation_succeeded')
        event_groups = {
            'jira': event_types
        }

    @staticmethod
    def get_scopes(event_context):
        issue = event_context['comment'].issue
        return get_issue_scopes(issue)


event_logger.register('jira_issue', IssueEventLogger)
event_logger.register('jira_comment', CommentEventLogger)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-06-09 17:30
import re

import django.core.validators
import django.utils.timezone
import django_fsm
import model_utils.fields
import waldur_core.core.fields
import waldur_core.core.validators
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('structure', '0001_squashed_0054'),
    ]

    operations = [
        migrations.CreateModel(
            name='Job',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created', model_utils.fields.AutoCreatedField(default=django.utils.timezone.now, editable=False, verbose_name='created')),
                ('modified', model_utils.fields.AutoLastModifiedField(default=django.utils.timezone.now, editable=False, verbose_name='modified')),
                ('description', models.CharField(blank=True, max_length=500, verbose_name='description')),
                ('name', models.CharField(max_length=150, validators=[waldur_core.core.validators.validate_name], verbose_name='name')),
                ('uuid', waldur_core.core.fields.UUIDField()),
                ('arguments', waldur_core.core.fields.JSONField(blank=True, default={}, null=True)),
                ('output', models.TextField(blank=True)),
                ('state', django_fsm.FSMIntegerField(choices=[(1, 'Scheduled'), (2, 'Executing'), (3, 'OK'), (4, 'Erred')], default=1)),
            ],
        ),
        migrations.CreateModel(
            name='Playbook',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('description', models.CharField(blank=True, max_length=500, verbose_name='description')),
                ('name', models.CharField(max_length=150, validators=[waldur_core.core.validators.validate_name], verbose_name='name')),
                ('uuid', waldur_core.core.fields.UUIDField()),
                ('workspace', models.CharField(help_text='Absolute path to the playbook workspace.', max_length=255, unique=True)),
                ('entrypoint', models.CharField(help_text='Relative path to the file in the workspace to execute.', max_length=255)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='PlaybookParameter',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('description', models.CharField(blank=True, max_length=500, verbose_name='description')),
                ('name', models.CharField(help_text='Required. 255 characters or fewer. Letters, numbers and _ characters', max_length=255, validators=[django.core.validators.RegexValidator(re.compile('^[\\w]+$'), 'Enter a valid name.')])),
                ('required', models.BooleanField(default=False)),
                ('default', models.CharField(blank=True, help_text='Default argument for this parameter.', max_length=255)),
                ('playbook', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='parameters', to='playbook_jobs.Playbook')),
            ],
        ),
        migrations.AddField(
            model_name='job',
            name='playbook',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='jobs', to='playbook_jobs.Playbook'),
        ),
        migrations.AddField(
            model_name='job',
            name='project',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='structure.Project'),
        ),
        migrations.AlterUniqueTogether(
            name='playbookparameter',
            unique_together=set([('playbook', 'name')]),
        ),
    ]



"""
WSGI config for Waldur Core.

It exposes the WSGI callable as a module-level variable named ``application``.

For more information on this file, see
https://docs.djangoproject.com/en/2.2/howto/deployment/wsgi/
"""

import os
import waldur_core  # noqa: F401 pre-load NC monkey-patching methods

os.environ.setdefault("DJANGO_SETTINGS_MODULE", "waldur_core.server.settings")

from django.core.wsgi import get_wsgi_application  # noqa: F402
application = get_wsgi_application()



import os
import re
import uuid

from django.conf import settings
from django.contrib.auth import get_user_model
from django.core import validators
from django.db import models
from django.utils.translation import ugettext_lazy as _
from model_utils import FieldTracker
from model_utils.models import TimeStampedModel

from waldur_ansible.common import models as common_models
from waldur_core.core import models as core_models, fields as core_fields
from waldur_openstack.openstack_tenant import models as openstack_models

from .backend import AnsiblePlaybookBackend

User = get_user_model()


def get_upload_path(instance, filename):
    return '%s/%s.png' % (instance._meta.model_name, instance.uuid.hex)


class Playbook(core_models.UuidMixin,
               core_models.NameMixin,
               core_models.DescribableMixin,
               models.Model):
    workspace = models.CharField(max_length=255, unique=True, help_text=_('Absolute path to the playbook workspace.'))
    entrypoint = models.CharField(max_length=255, help_text=_('Relative path to the file in the workspace to execute.'))
    image = models.ImageField(upload_to=get_upload_path, null=True, blank=True)
    tracker = FieldTracker()

    @staticmethod
    def get_url_name():
        return 'ansible_playbook'

    def get_playbook_path(self):
        return os.path.join(self.workspace, self.entrypoint)

    @staticmethod
    def generate_workspace_path():
        base_path = os.path.join(
            settings.MEDIA_ROOT,
            settings.WALDUR_PLAYBOOK_JOBS.get('PLAYBOOKS_DIR_NAME', 'ansible_playbooks'),
        )
        path = os.path.join(base_path, uuid.uuid4().hex)
        while os.path.exists(path):
            path = os.path.join(base_path, uuid.uuid4().hex)

        return path

    def get_backend(self):
        return AnsiblePlaybookBackend(self)

    def __str__(self):
        return self.name


class PlaybookParameter(core_models.DescribableMixin, models.Model):
    class Meta:
        unique_together = ('playbook', 'name')
        ordering = ['order']

    name = models.CharField(
        max_length=255,
        validators=[validators.RegexValidator(re.compile(r'^[\w]+$'), _('Enter a valid name.'))],
        help_text=_('Required. 255 characters or fewer. Letters, numbers and _ characters'),
    )
    playbook = models.ForeignKey(Playbook, on_delete=models.CASCADE, related_name='parameters')
    required = models.BooleanField(default=False)
    default = models.CharField(max_length=255, blank=True, help_text=_('Default argument for this parameter.'))
    order = models.PositiveIntegerField(default=0)

    def __str__(self):
        return self.name


class Job(core_models.UuidMixin,
          core_models.StateMixin,
          core_models.NameMixin,
          core_models.DescribableMixin,
          TimeStampedModel,
          common_models.ApplicationModel):
    class Meta:
        pass

    class Permissions:
        project_path = 'service_project_link__project'
        customer_path = 'service_project_link__project__customer'

    user = models.ForeignKey(on_delete=models.CASCADE, to=User, related_name='+')
    ssh_public_key = models.ForeignKey(on_delete=models.CASCADE, to=core_models.SshPublicKey, related_name='+')
    service_project_link = models.ForeignKey(on_delete=models.CASCADE, to=openstack_models.OpenStackTenantServiceProjectLink, related_name='+')
    subnet = models.ForeignKey(on_delete=models.CASCADE, to=openstack_models.SubNet, related_name='+')
    playbook = models.ForeignKey(on_delete=models.CASCADE, to=Playbook, related_name='jobs')
    arguments = core_fields.JSONField(default=dict, blank=True, null=True)
    output = models.TextField(blank=True)

    @staticmethod
    def get_url_name():
        return 'ansible_job'

    def get_backend(self):
        return self.playbook.get_backend()

    def __str__(self):
        return self.name

    def get_tag(self):
        return 'job:%s' % self.uuid.hex

    def get_related_resources(self):
        return openstack_models.Instance.objects.filter(tags__name=self.get_tag())



from django.contrib.contenttypes.models import ContentType

from waldur_core.cost_tracking import ConsumableItem
from waldur_core.cost_tracking.models import DefaultPriceListItem
from waldur_openstack.openstack import models as openstack_models


class Types:
    PACKAGE_TEMPLATE = 'PackageTemplate'


def get_consumable_item(package_template):
    return ConsumableItem(
        item_type=Types.PACKAGE_TEMPLATE,
        key=package_template.name,
        default_price=package_template.price / 24)  # default price per hour


def sync_price_list_item(package_template):
    resource_content_type = ContentType.objects.get_for_model(openstack_models.Tenant)
    consumable_item = get_consumable_item(package_template)
    DefaultPriceListItem._create_or_update_default_price_list_item(
        resource_content_type=resource_content_type,
        consumable_item=consumable_item,
    )



import logging

from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.utils.translation import ugettext_lazy as _
from rest_framework import exceptions as rf_exceptions
from rest_framework import status
from rest_framework.response import Response
from jira import JIRAError

from waldur_core.core import views as core_views
from waldur_mastermind.marketplace import models as marketplace_models
from waldur_mastermind.marketplace_support.utils import get_request_link, format_description
from waldur_mastermind.support import backend as support_backend
from waldur_mastermind.support import exceptions as support_exceptions
from waldur_mastermind.support import models as support_models
from waldur_mastermind.support import serializers as support_serializers
from waldur_mastermind.support import views as support_views

from . import serializers

logger = logging.getLogger(__name__)


def create_issue(order_item, description, summary):
    order_item_content_type = ContentType.objects.get_for_model(order_item)

    if not support_models.Issue.objects.filter(
            resource_object_id=order_item.id,
            resource_content_type=order_item_content_type).exists():
        issue_details = dict(
            caller=order_item.order.created_by,
            project=order_item.order.project,
            customer=order_item.order.project.customer,
            type=settings.WALDUR_SUPPORT['DEFAULT_OFFERING_ISSUE_TYPE'],
            description=description,
            summary=summary,
            resource=order_item)
        issue_details['summary'] = support_serializers.render_issue_template('summary', issue_details)
        issue_details['description'] = support_serializers.render_issue_template('description', issue_details)
        issue = support_models.Issue.objects.create(**issue_details)
        try:
            support_backend.get_active_backend().create_issue(issue)
        except support_exceptions.SupportUserInactive:
            issue.delete()
            order_item.resource.set_state_erred()
            order_item.resource.save(update_fields=['state'])
            raise rf_exceptions.ValidationError(_('Delete resource process is cancelled and issue not created '
                                                  'because a caller is inactive.'))
        else:
            ids = marketplace_models.OrderItem.objects.filter(resource=order_item.resource).\
                values_list('id', flat=True)
            linked_issues = support_models.Issue.objects.filter(resource_object_id__in=ids,
                                                                resource_content_type=order_item_content_type).\
                exclude(id=issue.id)
            try:
                support_backend.get_active_backend().create_issue_links(issue, list(linked_issues))
            except JIRAError as e:
                logger.exception('Linked issues have not been added: %s', e)
    else:
        message = 'An issue creating is skipped because an issue for order item %s exists already.' % order_item.uuid
        logger.warning(message)


class IssueViewSet(core_views.ActionsViewSet):
    def update(self, request, *args, **kwargs):
        uuid = request.data['uuid']
        order_item = marketplace_models.OrderItem.objects.get(uuid=uuid)
        summary = 'Request to switch plan for %s' % order_item.resource.scope.name
        request_url = get_request_link(order_item.resource.scope)
        description = format_description('UPDATE_RESOURCE_TEMPLATE', {
            'order_item': order_item,
            'request_url': request_url,
        })
        create_issue(order_item, description, summary)
        return Response(status=status.HTTP_202_ACCEPTED)

    def destroy(self, request, uuid, *args, **kwargs):
        order_item = marketplace_models.OrderItem.objects.get(uuid=uuid)
        summary = 'Request to terminate resource %s' % order_item.resource.scope.name
        request_url = get_request_link(order_item.resource.scope)
        description = format_description('TERMINATE_RESOURCE_TEMPLATE', {
            'order_item': order_item,
            'request_url': request_url,
        })
        create_issue(order_item, description, summary)
        return Response(status=status.HTTP_202_ACCEPTED)


class OfferingViewSet(support_views.OfferingViewSet):
    create_serializer_class = serializers.OfferingCreateSerializer



# -*- coding: utf-8 -*-
# Generated by Django 1.11.18 on 2019-02-04 16:01
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import re
import waldur_azure.validators


class Migration(migrations.Migration):

    dependencies = [
        ('waldur_azure', '0009_publicip_resource_group'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='sqldatabase',
            name='resource_group',
        ),
        migrations.AddField(
            model_name='sqlserver',
            name='fqdn',
            field=models.TextField(blank=True, null=True),
        ),
        migrations.AlterField(
            model_name='network',
            name='name',
            field=models.CharField(max_length=64, validators=[django.core.validators.RegexValidator(message='The name can contain only letters, numbers, underscore, period and hyphens.', regex=re.compile('^[a-zA-Z][a-zA-Z0-9._-]+$'))]),
        ),
        migrations.AlterField(
            model_name='networkinterface',
            name='name',
            field=models.CharField(max_length=80, validators=[django.core.validators.RegexValidator(message='The name can contain only letters, numbers, underscore, period and hyphens.', regex=re.compile('^[a-zA-Z][a-zA-Z0-9._-]+$'))]),
        ),
        migrations.AlterField(
            model_name='publicip',
            name='name',
            field=models.CharField(max_length=80, validators=[django.core.validators.RegexValidator(message='The name can contain only letters, numbers, underscore, period and hyphens.', regex=re.compile('^[a-zA-Z][a-zA-Z0-9._-]+$'))]),
        ),
        migrations.AlterField(
            model_name='sqldatabase',
            name='charset',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='sqldatabase',
            name='collation',
            field=models.CharField(blank=True, max_length=255, null=True),
        ),
        migrations.AlterField(
            model_name='sqldatabase',
            name='service_project_link',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='waldur_azure.AzureServiceProjectLink'),
        ),
        migrations.AlterField(
            model_name='sqlserver',
            name='name',
            field=models.CharField(max_length=80, validators=[django.core.validators.RegexValidator(message='The name can only be made up of lowercase letters "a"-"z", the numbers 0-9 and the hyphen. The hyphen may not lead or trail in the name.', regex=re.compile('^[a-z0-9][a-z0-9-]+[a-z0-9]$'))]),
        ),
        migrations.AlterField(
            model_name='sqlserver',
            name='password',
            field=models.CharField(max_length=128, validators=[django.core.validators.MinLengthValidator(8), django.core.validators.MaxLengthValidator(128), waldur_azure.validators.validate_password]),
        ),
        migrations.AlterField(
            model_name='sqlserver',
            name='service_project_link',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='waldur_azure.AzureServiceProjectLink'),
        ),
        migrations.AlterField(
            model_name='sqlserver',
            name='storage_mb',
            field=models.PositiveIntegerField(null=True, validators=[django.core.validators.MinValueValidator(5120), django.core.validators.MaxValueValidator(4194304)]),
        ),
        migrations.AlterField(
            model_name='sqlserver',
            name='username',
            field=models.CharField(max_length=50, validators=[waldur_azure.validators.SQLServerUsernameValidator]),
        ),
        migrations.AlterField(
            model_name='storageaccount',
            name='name',
            field=models.CharField(max_length=24, validators=[django.core.validators.RegexValidator(message='The name can contain only letters and numbers.', regex=re.compile('^[a-z][a-z0-9]{2,23}$'))]),
        ),
        migrations.AlterField(
            model_name='subnet',
            name='name',
            field=models.CharField(max_length=80, validators=[django.core.validators.RegexValidator(message='The name can contain only letters, numbers, underscore, period and hyphens.', regex=re.compile('^[a-zA-Z][a-zA-Z0-9._-]+$'))]),
        ),
        migrations.AlterField(
            model_name='virtualmachine',
            name='name',
            field=models.CharField(max_length=15, validators=[django.core.validators.RegexValidator(message='The name can contain only letters, numbers, and hyphens. The name must be shorter than 15 characters and start with a letter and must end with a letter or a number.', regex=re.compile('^[a-zA-Z][a-zA-Z0-9-]{0,13}[a-zA-Z0-9]$'))]),
        ),
        migrations.AlterField(
            model_name='virtualmachine',
            name='password',
            field=models.CharField(max_length=72, validators=[django.core.validators.MinLengthValidator(6), django.core.validators.MaxLengthValidator(72), waldur_azure.validators.validate_password, waldur_azure.validators.VirtualMachinePasswordValidator]),
        ),
    ]



class Register:
    _optimizers = {}
    _serializers = {}

    @classmethod
    def register_optimizer(cls, service_settings_type, optimizer):
        cls._optimizers[service_settings_type] = optimizer

    @classmethod
    def register_serializer(cls, service_settings_type, serializer):
        cls._serializers[service_settings_type] = serializer

    @classmethod
    def get_optimizer(cls, service_settings_type):
        return cls._optimizers.get(service_settings_type)

    @classmethod
    def get_serilizer(cls, service_settings_type):
        return cls._serializers.get(service_settings_type)



from django.utils.translation import ugettext_lazy as _
from rest_framework import status
from rest_framework.exceptions import APIException, ValidationError


class QuotaError(Exception):
    pass


class CreationConditionFailedQuotaError(QuotaError):
    pass


class QuotaExceededException(APIException):
    status_code = status.HTTP_409_CONFLICT
    default_detail = _('One or more quotas are over limit.')


class BackendQuotaUpdateError(APIException):
    status_code = status.HTTP_409_CONFLICT
    default_detail = _('It is impossible to modify backend quota through this endpoint.')


class QuotaValidationError(ValidationError):
    pass



# -*- coding: utf-8 -*-
# Generated by Django 1.11.1 on 2017-10-16 15:16
from django.conf import settings
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('waldur_slurm', '0002_add_gpu_ram_quotas'),
    ]

    operations = [
        migrations.CreateModel(
            name='AllocationUsage',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('username', models.CharField(max_length=32)),
                ('year', models.PositiveSmallIntegerField()),
                ('month', models.PositiveSmallIntegerField(validators=[django.core.validators.MinValueValidator(1), django.core.validators.MaxValueValidator(12)])),
                ('cpu_usage', models.IntegerField(default=0)),
                ('ram_usage', models.IntegerField(default=0)),
                ('gpu_usage', models.IntegerField(default=0)),
                ('allocation', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='waldur_slurm.Allocation')),
                ('user', models.OneToOneField(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'ordering': ['allocation'],
            },
        ),
    ]



import logging

from django_filters.rest_framework import DjangoFilterBackend
from rest_framework import generics, permissions, viewsets

from waldur_core.core import mixins as core_mixins
from waldur_core.structure import filters as structure_filters
from waldur_core.structure import permissions as structure_permissions
from waldur_core.structure import views as structure_views

from . import filters, executors, models, serializers

logger = logging.getLogger(__name__)


class JiraServiceViewSet(structure_views.BaseServiceViewSet):
    queryset = models.JiraService.objects.all()
    serializer_class = serializers.ServiceSerializer


class JiraServiceProjectLinkViewSet(structure_views.BaseServiceProjectLinkViewSet):
    queryset = models.JiraServiceProjectLink.objects.all()
    serializer_class = serializers.ServiceProjectLinkSerializer


class JiraPermissionMixin:
    def get_queryset(self):
        user = self.request.user
        queryset = super(JiraPermissionMixin, self).get_queryset()
        if user.is_staff:
            return queryset
        else:
            return queryset.filter(user=user)


class ProjectTemplateViewSet(structure_views.BaseServicePropertyViewSet):
    queryset = models.ProjectTemplate.objects.all()
    filterset_class = filters.ProjectTemplateFilter
    serializer_class = serializers.ProjectTemplateSerializer
    lookup_field = 'uuid'


class ProjectViewSet(structure_views.ImportableResourceViewSet):
    queryset = models.Project.objects.all()
    filterset_class = filters.ProjectFilter
    serializer_class = serializers.ProjectSerializer
    create_executor = executors.ProjectCreateExecutor
    update_executor = executors.ProjectUpdateExecutor
    delete_executor = executors.ProjectDeleteExecutor
    async_executor = False

    destroy_permissions = [structure_permissions.is_staff]

    importable_resources_backend_method = 'get_resources_for_import'
    importable_resources_serializer_class = serializers.ProjectImportableSerializer
    import_resource_serializer_class = serializers.ProjectImportSerializer


class IssueTypeViewSet(structure_views.BaseServicePropertyViewSet):
    queryset = models.IssueType.objects.all()
    filterset_class = filters.IssueTypeFilter
    serializer_class = serializers.IssueTypeSerializer
    lookup_field = 'uuid'


class PriorityViewSet(structure_views.BaseServicePropertyViewSet):
    queryset = models.Priority.objects.all()
    serializer_class = serializers.PrioritySerializer
    filterset_class = filters.PriorityFilter
    lookup_field = 'uuid'


class IssueViewSet(JiraPermissionMixin,
                   structure_views.ResourceViewSet):
    queryset = models.Issue.objects.all()
    filterset_class = filters.IssueFilter
    serializer_class = serializers.IssueSerializer
    create_executor = executors.IssueCreateExecutor
    update_executor = executors.IssueUpdateExecutor
    delete_executor = executors.IssueDeleteExecutor
    async_executor = False


class CommentViewSet(JiraPermissionMixin,
                     structure_views.ResourceViewSet):
    queryset = models.Comment.objects.all()
    filterset_class = filters.CommentFilter
    serializer_class = serializers.CommentSerializer
    create_executor = executors.CommentCreateExecutor
    update_executor = executors.CommentUpdateExecutor
    delete_executor = executors.CommentDeleteExecutor
    async_executor = False


class AttachmentViewSet(JiraPermissionMixin,
                        core_mixins.CreateExecutorMixin,
                        core_mixins.DeleteExecutorMixin,
                        viewsets.ModelViewSet):
    queryset = models.Attachment.objects.all()
    filterset_class = filters.AttachmentFilter
    filter_backends = structure_filters.GenericRoleFilter, DjangoFilterBackend
    permission_classes = permissions.IsAuthenticated, permissions.DjangoObjectPermissions
    serializer_class = serializers.AttachmentSerializer
    create_executor = executors.AttachmentCreateExecutor
    delete_executor = executors.AttachmentDeleteExecutor
    async_executor = False
    lookup_field = 'uuid'


class WebHookReceiverViewSet(generics.CreateAPIView):
    authentication_classes = ()
    permission_classes = ()
    serializer_class = serializers.WebHookReceiverSerializer

    def create(self, request, *args, **kwargs):
        try:
            return super(WebHookReceiverViewSet, self).create(request, *args, **kwargs)
        except Exception as e:
            # Throw validation errors to the logs
            logger.error("Can't parse JIRA WebHook request: %s" % e)
            raise


def get_jira_projects_count(project):
    return project.quotas.get(name='nc_jira_project_count').usage


structure_views.ProjectCountersView.register_counter('jira-projects', get_jira_projects_count)



from django.core.exceptions import ObjectDoesNotExist
from django.core.management.base import BaseCommand, CommandError

from waldur_mastermind.marketplace.models import Offering, Plan
from waldur_mastermind.marketplace_openstack import PACKAGE_TYPE

from ... import utils


class Command(BaseCommand):
    help = """Merge marketplace plans related to OpenStack tenant offering."""

    def add_arguments(self, parser):
        parser.add_argument('--offering', dest='offering_uuid', required=True,
                            help='UUID of marketplace offering for OpenStack tenant provisioning.')
        parser.add_argument('--plan', dest='plan_uuid', required=True,
                            help='UUID of example marketplace plan related to the same offering.')

    def handle(self, *args, **options):
        offering_uuid = options['offering_uuid']
        plan_uuid = options['plan_uuid']

        try:
            offering = Offering.objects.get(uuid=offering_uuid)
        except ObjectDoesNotExist:
            raise CommandError('Offering with given UUID is not found.')

        if offering.type != PACKAGE_TYPE:
            raise CommandError('Offering is not related to OpenStack tenants.')

        try:
            plan = Plan.objects.get(uuid=plan_uuid)
        except ObjectDoesNotExist:
            raise CommandError('Plan with given UUID is not found.')

        if plan.offering != offering:
            raise CommandError('Plan is related to another offering.')

        utils.merge_plans(offering, plan)



from waldur_core.core import executors as core_executors
from waldur_core.core import utils as core_utils

from . import tasks


class TerminateResourceExecutor(core_executors.BaseExecutor):
    @classmethod
    def get_task_signature(cls, instance, serialized_instance, user, **kwargs):
        serialized_user = core_utils.serialize_instance(user)
        return tasks.terminate_resource.si(serialized_instance, serialized_user)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2017-12-27 08:36
from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('waldur_jira', '0005_drop_impact_field'),
    ]

    operations = [
        migrations.AddField(
            model_name='issue',
            name='parent',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='waldur_jira.Issue'),
        ),
        migrations.AddField(
            model_name='issuetype',
            name='subtask',
            field=models.BooleanField(default=False),
        ),
    ]



from waldur_mastermind.marketplace import processors

from .utils import DockerExecutorMixin

"""
It is expected that offering plugin_options field is dict with following structure:

language: python

environ:
    USERNAME: admin
    PASSWORD: secret

create:
    import os
    print("Creating resource ", os.environ.get('RESOURCE_NAME'))

update:
    import os
    print("Updating resource ", os.environ.get('RESOURCE_NAME'))

delete:
    import os
    print("Deleting resource ", os.environ.get('RESOURCE_NAME'))

pull:
    import os
    print("Pulling resource ", os.environ.get('RESOURCE_NAME'))
"""


class CreateProcessor(DockerExecutorMixin, processors.AbstractCreateResourceProcessor):
    hook_type = 'create'


class UpdateProcessor(DockerExecutorMixin, processors.AbstractUpdateResourceProcessor):
    hook_type = 'update'


class DeleteProcessor(DockerExecutorMixin, processors.AbstractDeleteResourceProcessor):
    hook_type = 'delete'



from django.conf.urls import url

from waldur_mastermind.marketplace import views


def register_in(router):
    router.register(r'marketplace-service-providers', views.ServiceProviderViewSet,
                    basename='marketplace-service-provider'),
    router.register(r'marketplace-categories', views.CategoryViewSet,
                    basename='marketplace-category'),
    router.register(r'marketplace-offerings', views.OfferingViewSet,
                    basename='marketplace-offering')
    router.register(r'marketplace-plans', views.PlanViewSet,
                    basename='marketplace-plan')
    router.register(r'marketplace-screenshots', views.ScreenshotViewSet,
                    basename='marketplace-screenshot')
    router.register(r'marketplace-cart-items', views.CartItemViewSet,
                    basename='marketplace-cart-item')
    router.register(r'marketplace-orders', views.OrderViewSet,
                    basename='marketplace-order')
    router.register(r'marketplace-order-items', views.OrderItemViewSet,
                    basename='marketplace-order-item')
    router.register(r'marketplace-resources', views.ResourceViewSet,
                    basename='marketplace-resource')
    router.register(r'marketplace-category-component-usages', views.CategoryComponentUsageViewSet,
                    basename='marketplace-category-component-usage')
    router.register(r'marketplace-component-usages', views.ComponentUsageViewSet,
                    basename='marketplace-component-usage')
    router.register(r'marketplace-public-api', views.MarketplaceAPIViewSet,
                    basename='marketplace-public-api'),
    router.register(r'marketplace-offering-files', views.OfferingFileViewSet,
                    basename='marketplace-offering-file'),


urlpatterns = [
    url(r'^api/customers/(?P<uuid>[a-f0-9]+)/offerings/$', views.CustomerOfferingViewSet.as_view()),
    url(r'^api/marketplace-plugins/$', views.PluginViewSet.as_view()),
]



# -*- coding: utf-8 -*-
# Generated by Django 1.9 on 2017-05-26 13:30
from django.conf import settings
import django.core.validators
from django.db import migrations, models
import django.db.models.deletion
import django.utils.timezone
import waldur_core.core.fields
import re
import waldur_freeipa.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name='Profile',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('uuid', waldur_core.core.fields.UUIDField()),
                ('username', models.CharField(help_text='Letters, numbers and ./+/-/_ characters', max_length=255, unique=True, validators=[waldur_freeipa.models.validate_username, django.core.validators.RegexValidator(re.compile(r'^[a-zA-Z0-9_.][a-zA-Z0-9_.-]*[a-zA-Z0-9_.$-]?$'), 'Enter a valid username.', 'invalid')], verbose_name='username')),
                ('agreement_date', models.DateTimeField(default=django.utils.timezone.now, help_text='Indicates when the user has agreed with the policy.', verbose_name='agreement date')),
                ('is_active', models.BooleanField(default=True, verbose_name='active')),
                ('user', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, to=settings.AUTH_USER_MODEL)),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



import logging
import requests

from django.conf import settings

from waldur_core.structure import ServiceBackend

from . import exceptions

logger = logging.getLogger(__name__)


class DataciteBackend(ServiceBackend):
    def __init__(self):
        self.settings = settings.WALDUR_PID['DATACITE']

    def post(self, data):
        headers = {
            'Content-Type': 'application/vnd.api+json',
        }

        if not self.settings['API_URL']:
            raise exceptions.DataciteException('API_URL is not defined.')

        response = requests.post(
            self.settings['API_URL'],
            headers=headers,
            json=data,
            auth=(self.settings['REPOSITORY_ID'], self.settings['PASSWORD'])
        )

        return response

    def create_doi(self, instance):
        data = {
            'data': {
                'type': 'dois',
                'attributes': {
                    'prefix': self.settings['PREFIX'],
                    'event': 'publish',
                    'creators': [{
                        'name': instance.get_datacite_creators_name()
                    }],
                    'titles': [{
                        'title': instance.get_datacite_title()
                    }],
                    'descriptions': [{'description': instance.get_datacite_description()}],
                    'publisher': self.settings['PUBLISHER'],
                    'publicationYear': instance.get_datacite_publication_year(),
                    'types': {
                        'resourceTypeGeneral': 'Service'
                    },
                    'url': instance.get_datacite_url(),
                    'schemaVersion': 'http://datacite.org/schema/kernel-4'
                }
            }
        }
        response = self.post(data)

        if response.status_code == 201:
            instance.datacite_doi = response.json()['data']['id']
            instance.save()
        else:
            logger.error('Creating Datacite DOI for %s has failed. Status code: %s, message: %s.' % (
                instance,
                response.status_code,
                response.text
            ))



from decimal import Decimal

from django.core.validators import MinValueValidator
from django.db import models
from django.utils.translation import ugettext_lazy as _


class UnitPriceMixin(models.Model):
    """
    Mixin to expose standardized "unit_price" and "unit" field.
    """
    class Meta:
        abstract = True

    class Units:
        PER_MONTH = 'month'
        PER_HALF_MONTH = 'half_month'
        PER_DAY = 'day'
        PER_HOUR = 'hour'
        QUANTITY = 'quantity'

        CHOICES = (
            (PER_MONTH, _('Per month')),
            (PER_HALF_MONTH, _('Per half month')),
            (PER_DAY, _('Per day')),
            (PER_HOUR, _('Per hour')),
            (QUANTITY, _('Quantity')),
        )

    unit_price = models.DecimalField(default=0, max_digits=22, decimal_places=7,
                                     validators=[MinValueValidator(Decimal('0'))])
    unit = models.CharField(default=Units.PER_DAY, max_length=30, choices=Units.CHOICES)


class ProductCodeMixin(models.Model):
    class Meta:
        abstract = True

    # technical code used by accounting software
    product_code = models.CharField(max_length=30, blank=True)
    # article code is used for encoding product category in accounting software
    article_code = models.CharField(max_length=30, blank=True)



import logging

from django.contrib.contenttypes.fields import GenericForeignKey
from django.contrib.contenttypes.models import ContentType
from django.contrib.postgres.fields import JSONField
from django.db import models
from django.utils.translation import ugettext_lazy as _
from model_utils import FieldTracker

from waldur_core.core import models as core_models
from waldur_core.structure import models as structure_models
from waldur_core.structure.models import NewResource, ServiceSettings

logger = logging.getLogger(__name__)


class RancherService(structure_models.Service):
    projects = models.ManyToManyField(
        structure_models.Project, related_name='rancher_services', through='RancherServiceProjectLink')

    class Meta:
        unique_together = ('customer', 'settings')
        verbose_name = _('Rancher provider')
        verbose_name_plural = _('Rancher providers')

    @classmethod
    def get_url_name(cls):
        return 'rancher'


class RancherServiceProjectLink(structure_models.ServiceProjectLink):

    service = models.ForeignKey(on_delete=models.CASCADE, to=RancherService)

    class Meta(structure_models.ServiceProjectLink.Meta):
        verbose_name = _('Rancher provider project link')
        verbose_name_plural = _('Rancher provider project links')

    @classmethod
    def get_url_name(cls):
        return 'rancher-spl'


class Cluster(NewResource):
    service_project_link = models.ForeignKey(
        RancherServiceProjectLink, related_name='k8s_clusters', on_delete=models.PROTECT)

    """
    Rancher generated node installation command base. For example:
    sudo docker run -d --privileged --restart=unless-stopped --net=host
    -v /etc/kubernetes:/etc/kubernetes -v /var/run:/var/run rancher/rancher-agent:v2.2.8
    --server https://192.168.33.13
    --token df8vrttmcmz8qzfbp74t6nkl5t5pbkrjh8wgkv27zrk8ldhfj6sp4w
    --ca-checksum e3596989da2fa5f8a7bdfbfd1079f87033217152db4dfc93532932b17aad1567
    --etcd --controlplane --worker
    """
    node_command = models.CharField(max_length=1024, blank=True,
                                    help_text='Rancher generated node installation command base.')
    tracker = FieldTracker()
    tenant_settings = models.ForeignKey(
        to=structure_models.ServiceSettings,
        on_delete=models.CASCADE,
        null=True,
        blank=True,
    )
    runtime_state = models.CharField(max_length=255, blank=True)

    class Meta:
        unique_together = (('service_project_link', 'backend_id'), ('service_project_link', 'name'))

    @classmethod
    def get_url_name(cls):
        return 'rancher-cluster'

    def get_access_url(self):
        return self.service_project_link.service.settings.backend_url

    def __str__(self):
        return self.name


class Node(core_models.UuidMixin,
           core_models.NameMixin,
           structure_models.StructureModel,
           core_models.StateMixin,
           structure_models.TimeStampedModel):
    content_type = models.ForeignKey(on_delete=models.CASCADE, to=ContentType, null=True, related_name='+')
    object_id = models.PositiveIntegerField(null=True)
    instance = GenericForeignKey('content_type', 'object_id')  # a virtual machine where will deploy k8s node.
    cluster = models.ForeignKey(Cluster, on_delete=models.CASCADE)
    controlplane_role = models.BooleanField(default=False)
    etcd_role = models.BooleanField(default=False)
    worker_role = models.BooleanField(default=False)
    backend_id = models.CharField(max_length=255, blank=True)
    initial_data = JSONField(blank=True,
                             default=dict,
                             help_text=_('Initial data for instance creating.'))
    runtime_state = models.CharField(max_length=255, blank=True)
    k8s_version = models.CharField(max_length=255, blank=True)
    docker_version = models.CharField(max_length=255, blank=True)
    cpu_allocated = models.FloatField(blank=True, null=True)
    cpu_total = models.IntegerField(blank=True, null=True)
    ram_allocated = models.IntegerField(blank=True, null=True, help_text='Allocated RAM in Mi.')
    ram_total = models.IntegerField(blank=True, null=True, help_text='Total RAM in Mi.')
    pods_allocated = models.IntegerField(blank=True, null=True)
    pods_total = models.IntegerField(blank=True, null=True)
    labels = JSONField(blank=True, default=dict)
    annotations = JSONField(blank=True, default=dict)

    tracker = FieldTracker()

    def get_node_command(self):
        roles_command = []
        if self.controlplane_role:
            roles_command.append('--controlplane')

        if self.etcd_role:
            roles_command.append('--etcd')

        if self.worker_role:
            roles_command.append('--worker')

        return self.cluster.node_command + ' ' + ' '.join(roles_command)

    class Meta:
        unique_together = (('content_type', 'object_id'), ('cluster', 'name'))

    class Permissions:
        customer_path = 'cluster__service_project_link__project__customer'
        project_path = 'cluster__service_project_link__project'
        service_path = 'cluster__service_project_link__service'

    @classmethod
    def get_url_name(cls):
        return 'rancher-node'

    @property
    def service_project_link(self):
        return self.cluster.service_project_link

    def __str__(self):
        return self.name


class RancherUser(models.Model):
    user = models.ForeignKey(core_models.User, on_delete=models.PROTECT)
    clusters = models.ManyToManyField(Cluster, through='RancherUserClusterLink')
    settings = models.ForeignKey('structure.ServiceSettings', on_delete=models.PROTECT)
    backend_id = models.CharField(max_length=255, blank=True)
    is_active = models.BooleanField(default=True)

    @staticmethod
    def make_random_password():
        return core_models.User.objects.make_random_password()

    class Meta:
        unique_together = (('user', 'settings'),)

    def __str__(self):
        return self.user.username


class ClusterRole(models.CharField):
    CLUSTER_OWNER = 'owner'
    CLUSTER_MEMBER = 'member'

    CHOICES = (
        (CLUSTER_OWNER, 'Cluster owner'),
        (CLUSTER_MEMBER, 'Cluster member'),
    )

    def __init__(self, *args, **kwargs):
        kwargs['max_length'] = 30
        kwargs['choices'] = self.CHOICES
        super(ClusterRole, self).__init__(*args, **kwargs)


class RancherUserClusterLink(models.Model):
    user = models.ForeignKey(RancherUser, on_delete=models.CASCADE)
    cluster = models.ForeignKey(Cluster, on_delete=models.CASCADE)
    backend_id = models.CharField(max_length=255, blank=True)
    role = ClusterRole(db_index=True)

    class Meta:
        unique_together = (('user', 'cluster', 'role'),)


class Catalog(core_models.UuidMixin,
              core_models.NameMixin,
              core_models.DescribableMixin,
              structure_models.TimeStampedModel,
              core_models.RuntimeStateMixin):
    # Rancher supports global, cluster and project scope
    content_type = models.ForeignKey(on_delete=models.CASCADE, to=ContentType, null=True, related_name='+')
    object_id = models.PositiveIntegerField(null=True)
    scope = GenericForeignKey('content_type', 'object_id')
    backend_id = models.CharField(max_length=255, blank=True)
    catalog_url = models.URLField()
    branch = models.CharField(max_length=255)
    commit = models.CharField(max_length=40, blank=True)
    username = models.CharField(max_length=255, blank=True)
    password = models.CharField(max_length=255, blank=True)

    def get_backend(self):
        return self.scope.get_backend()

    @property
    def scope_type(self):
        if isinstance(self.scope, ServiceSettings):
            return 'global'
        elif isinstance(self.scope, Cluster):
            return 'cluster'
        else:
            return 'project'



from urllib.parse import urlparse
import uuid

from django.contrib.contenttypes.models import ContentType
from django.forms.fields import MultipleChoiceField
from django.urls import resolve
import django_filters
from django_filters.constants import EMPTY_VALUES
from django_filters.filters import MultipleChoiceFilter
from django_filters.rest_framework import DjangoFilterBackend
from rest_framework.filters import BaseFilterBackend

from waldur_core.core import serializers as core_serializers, fields as core_fields, models as core_models


class GenericKeyFilterBackend(DjangoFilterBackend):
    """
    Backend for filtering by backend field.

    Methods 'get_related_models' and 'get_field_name' has to be implemented.
    Example:

        class ScopeFilterBackend(core_filters.GenericKeyFilterBackend):

            def get_related_models(self):
                return utils.get_loggable_models()

            def get_field_name(self):
                return 'scope'
    """
    content_type_field = 'content_type'
    object_id_field = 'object_id'

    def get_related_models(self):
        """ Return all models that are acceptable as filter argument """
        raise NotImplementedError

    def get_field_name(self):
        """ Get name of filter field name in request """
        raise NotImplementedError

    def get_field_value(self, request):
        field_name = self.get_field_name()
        return request.query_params.get(field_name)

    def filter_queryset(self, request, queryset, view):
        value = self.get_field_value(request)
        if value:
            field = core_serializers.GenericRelatedField(related_models=self.get_related_models())
            # Trick to set field context without serializer
            field._context = {'request': request}
            obj = field.to_internal_value(value)
            ct = ContentType.objects.get_for_model(obj)
            return queryset.filter(**{self.object_id_field: obj.id, self.content_type_field: ct})
        return queryset


class MappedFilterMixin:

    def __init__(self, choice_mappings, **kwargs):
        super(MappedFilterMixin, self).__init__(**kwargs)

        # TODO: enable this assert then filtering by numbers will be disabled
        # assert set(k for k, _ in self.field.choices) == set(choice_mappings.keys()), 'Choices do not match mappings'
        assert len(set(choice_mappings.values())) == len(choice_mappings), 'Mappings are not unique'

        self.mapped_to_model = choice_mappings
        self.model_to_mapped = {v: k for k, v in choice_mappings.items()}


class MappedChoiceFilter(MappedFilterMixin, django_filters.ChoiceFilter):
    """
    A choice field that maps enum values from representation to model ones and back.

    Filter analog for MappedChoiceField.
    """

    def filter(self, qs, value):
        if value in self.mapped_to_model:
            value = self.mapped_to_model[value]
        return super(MappedChoiceFilter, self).filter(qs, value)


class MappedMultipleChoiceFilter(MappedFilterMixin, django_filters.MultipleChoiceFilter):
    """
    A multiple choice field that maps enum values from representation to model ones and back.

    Filter analog for MappedChoiceField that allow to filter by several choices.
    """

    def filter(self, qs, value):
        value = [self.mapped_to_model[v] for v in value if v in self.mapped_to_model]
        return super(MappedMultipleChoiceFilter, self).filter(qs, value)


class LooseMultipleChoiceField(MultipleChoiceField):
    def valid_value(self, value):
        return True


class LooseMultipleChoiceFilter(MultipleChoiceFilter):
    """
    A multiple choice filter field that skips validation of values.
    Based on https://github.com/carltongibson/django-filter/issues/137#issuecomment-37820702
    """
    field_class = LooseMultipleChoiceField


class StateFilter(MappedMultipleChoiceFilter):

    DEFAULT_CHOICES = (
        ('Creation Scheduled', 'Creation Scheduled'),
        ('Creating', 'Creating'),
        ('Update Scheduled', 'Update Scheduled'),
        ('Updating', 'Updating'),
        ('Deletion Scheduled', 'Deletion Scheduled'),
        ('Deleting', 'Deleting'),
        ('OK', 'OK'),
        ('Erred', 'Erred'),
    )

    States = core_models.StateMixin.States
    DEFAULT_CHOICE_MAPPING = {
        'Creation Scheduled': States.CREATION_SCHEDULED,
        'Creating': States.CREATING,
        'Update Scheduled': States.UPDATE_SCHEDULED,
        'Updating': States.UPDATING,
        'Deletion Scheduled': States.DELETION_SCHEDULED,
        'Deleting': States.DELETING,
        'OK': States.OK,
        'Erred': States.ERRED,
    }

    def __init__(self, choices=DEFAULT_CHOICES, choice_mappings=None, **kwargs):
        if choice_mappings is None:
            choice_mappings = self.DEFAULT_CHOICE_MAPPING
        super(StateFilter, self).__init__(choices=choices, choice_mappings=choice_mappings, **kwargs)


class URLFilter(django_filters.CharFilter):
    """ Filter by hyperlinks. ViewSet name must be supplied in order to validate URL. """

    def __init__(self, view_name, lookup_field='uuid', **kwargs):
        super(URLFilter, self).__init__(**kwargs)
        self.view_name = view_name
        self.lookup_field = lookup_field

    def get_uuid(self, value):
        uuid_value = ''
        path = urlparse(value).path
        if path.startswith('/'):
            match = resolve(path)
            if match.url_name == self.view_name:
                uuid_value = match.kwargs.get(self.lookup_field)
        return uuid_value

    def filter(self, qs, value):
        if value in EMPTY_VALUES:
            return qs

        uuid_value = self.get_uuid(value)
        try:
            uuid.UUID(uuid_value)
        except ValueError:
            return qs.none()
        return super(URLFilter, self).filter(qs, uuid_value)


class TimestampFilter(django_filters.NumberFilter):
    """
    Filter for dates in timestamp format
    """

    def filter(self, qs, value):
        if value in EMPTY_VALUES:
            return qs

        field = core_fields.TimestampField()
        datetime_value = field.to_internal_value(value)
        return super(TimestampFilter, self).filter(qs, datetime_value)


class CategoryFilter(django_filters.CharFilter):
    """
    Filters queryset by category names.
    If category name does not match, it will work as CharFilter.

    :param categories: dictionary of category names as keys and category elements as values.
    """

    def __init__(self, categories, **kwargs):
        super(CategoryFilter, self).__init__(**kwargs)
        self.categories = categories

    def filter(self, qs, value):
        if value in self.categories.keys():
            return qs.filter(**{'%s__in' % self.name: self.categories[value]})

        return super(CategoryFilter, self).filter(qs, value)


class StaffOrUserFilter(BaseFilterBackend):
    def filter_queryset(self, request, queryset, view):
        if request.user.is_staff or request.user.is_support:
            return queryset

        return queryset.filter(user=request.user)


class ContentTypeFilter(django_filters.CharFilter):

    def filter(self, qs, value):
        if value in EMPTY_VALUES:
            return qs
        try:
            app_label, model = value.split('.')
            ct = ContentType.objects.get(app_label=app_label, model=model)
            return super(ContentTypeFilter, self).filter(qs, ct)
        except (ContentType.DoesNotExist, ValueError):
            return qs.none()


class ExternalFilterBackend(BaseFilterBackend):
    """
    Support external filters registered in other apps
    """

    @classmethod
    def get_registered_filters(cls):
        return getattr(cls, '_filters', [])

    @classmethod
    def register(cls, external_filter):
        assert isinstance(external_filter, BaseFilterBackend), 'Registered filter has to inherit BaseFilterBackend'
        if hasattr(cls, '_filters'):
            cls._filters.append(external_filter)
        else:
            cls._filters = [external_filter]

    def filter_queryset(self, request, queryset, view):
        for item in self.__class__.get_registered_filters():
            queryset = item.filter_queryset(request, queryset, view)
        return queryset


class SummaryFilter(DjangoFilterBackend):
    """ Base filter for summary querysets """

    def filter_queryset(self, request, queryset, view):
        queryset = self.filter(request, queryset, view)
        return queryset

    def get_queryset_filter(self, queryset):
        """ Return specific for queryset filter if it exists """
        raise NotImplementedError()

    def get_base_filter(self):
        """ Return base filter that could be used for all summary objects """
        raise NotImplementedError()

    def _get_filter(self, queryset):
        try:
            return self.get_queryset_filter(queryset)
        except NotImplementedError:
            return self.get_base_filter()

    def filter(self, request, queryset, view):
        """ Filter each resource separately using its own filter """
        summary_queryset = queryset
        filtered_querysets = []
        for queryset in summary_queryset.querysets:
            filter_class = self._get_filter(queryset)
            queryset = filter_class(request.query_params, queryset=queryset).qs
            filtered_querysets.append(queryset)

        summary_queryset.querysets = filtered_querysets
        return summary_queryset


class EmptyFilter(django_filters.CharFilter):
    """
    This filter always returns empty queryset for non-empty value.
    It is used when model does not support particular filter field yet it
    should not simply ignore unknown field and instead should return empty queryset.
    """
    def filter(self, qs, value):
        if value in EMPTY_VALUES:
            return qs
        else:
            return qs.none()


class BaseSummaryFilterSet(django_filters.FilterSet):

    def filter_queryset(self, queryset):
        # Skip queryset class validation
        for name, value in self.form.cleaned_data.items():
            queryset = self.filters[name].filter(queryset, value)
        return queryset



from django.apps import AppConfig


class MarketplaceChecklistConfig(AppConfig):
    name = 'waldur_mastermind.marketplace_checklist'
    verbose_name = 'Marketplace Checklist'

    def ready(self):
        pass



from waldur_core.core import WaldurExtension


class RancherExtension(WaldurExtension):

    class Settings:
        WALDUR_RANCHER = {
            'ROLE_REQUIREMENT': {
                'controllplane': {'CPU': 2, 'RAM': 4096},
                'etcd': {'CPU': 2, 'RAM': 4096},
                'worker': {'CPU': 1, 'RAM': 1024},
            },
            'MOUNT_POINT_CHOICES': (
                '/var/lib/docker',
                '/var/lib/etcd',
                '/opt/media01',
            ),
            'MOUNT_POINT_MIN_SIZE': {
                '/var/lib/docker': 64,
                '/var/lib/etcd': 10,
            },
            'SYSTEM_VOLUME_MIN_SIZE': 64,
            'ACTIVE_NODE_STATE': 'active',
            'ACTIVE_CLUSTER_STATE': 'active',
        }

    @staticmethod
    def django_app():
        return 'waldur_rancher'

    @staticmethod
    def rest_urls():
        from .urls import register_in
        return register_in

    @staticmethod
    def celery_tasks():
        from datetime import timedelta
        return {
            'waldur-rancher-update-clusters-nodes': {
                'task': 'waldur_rancher.update_clusters_nodes',
                'schedule': timedelta(hours=24),
                'args': (),
            },
            'waldur-rancher-sync-users': {
                'task': 'waldur_rancher.sync_users',
                'schedule': timedelta(hours=24),
                'args': (),
            },
        }

    @staticmethod
    def get_public_settings():
        return [
            'ROLE_REQUIREMENT',
            'MOUNT_POINT_CHOICES',
            'MOUNT_POINT_MIN_SIZE',
            'SYSTEM_VOLUME_MIN_SIZE',
        ]



from django.apps import apps
from django.db import models
from django.utils.lru_cache import lru_cache

from waldur_core.core import models as core_models
from waldur_core.structure import models as structure_models


class OutputMixin(models.Model):
    output = models.TextField(blank=True)

    class Meta:
        abstract = True


class UuidStrMixin(core_models.UuidMixin):

    class Meta:
        abstract = True

    def __str__(self):
        return '%s: %s' % (self.__class__.__name__, self.uuid.hex)


class ApplicationModel(structure_models.StructureModel):
    class Meta:
        abstract = True

    @classmethod
    @lru_cache(maxsize=1)
    def get_application_models(cls):
        return [model for model in apps.get_models() if issubclass(model, cls)]
